@@ -30,7 +30,7 @@
 
 #include <config.h>
 
-#include "nemo-application.h"
+#include "nemo-main-application.h"
 
 #include <libnemo-private/nemo-debug.h>
 #include <eel/eel-debug.h>
@@ -75,8 +75,6 @@ main (int argc, char *argv[])
 	mallopt (M_MMAP_THRESHOLD, 128 *1024);
 #endif
 
-	g_type_init ();
-
 	/* This will be done by gtk+ later, but for now, force it to GNOME */
 	g_desktop_app_info_set_desktop_env ("GNOME");
 
@@ -96,7 +94,12 @@ main (int argc, char *argv[])
 #endif
 
 	/* Run the nemo application. */
-	application = nemo_application_get_singleton ();
+	application = nemo_main_application_get_singleton ();
+
+    /* hold indefinitely if we're asked to persist */
+    if (g_getenv ("NEMO_PERSIST") != NULL) {
+        g_application_hold (G_APPLICATION (application));
+    }
 
 	retval = g_application_run (G_APPLICATION (application),
 				    argc, argv);
