@@ -1,4 +1,4 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/* -*- Mode: C; indent-tabs-mode: f; c-basic-offset: 4; tab-width: 4 -*- */
 
 /* nemo-mime-actions.c - uri-specific versions of mime action functions
 
@@ -27,6 +27,8 @@
 #include "nemo-mime-actions.h"
 
 #include "nemo-window-slot.h"
+#include "nemo-error-reporting.h"
+#include "nemo-desktop-window.h"
 
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-stock-dialogs.h>
@@ -44,6 +46,14 @@
 #include <libnemo-private/nemo-desktop-icon-file.h>
 #include <libnemo-private/nemo-global-preferences.h>
 #include <libnemo-private/nemo-signaller.h>
+#include <libnemo-private/nemo-mime-application-chooser.h>
+#include <sys/stat.h>
+
+enum {
+    UNIX_PERM_USER_EXEC = S_IXUSR,
+    UNIX_PERM_GROUP_EXEC = S_IXGRP,
+    UNIX_PERM_OTHER_EXEC = S_IXOTH,
+};
 
 #define DEBUG_FLAG NEMO_DEBUG_MIME
 #include <libnemo-private/nemo-debug.h>
@@ -93,6 +103,7 @@ typedef struct {
 #define RESPONSE_DISPLAY 1001
 #define RESPONSE_RUN_IN_TERMINAL 1002
 #define RESPONSE_MARK_TRUSTED 1003
+#define RESPONSE_OPEN_WITH 1004
 
 #define SILENT_WINDOW_OPEN_LIMIT 5
 #define SILENT_OPEN_LIMIT 5
@@ -305,7 +316,7 @@ file_has_local_path (NemoFile *file)
 	
 	/* Don't only check _is_native, because we want to support
 	   using the fuse path */
-	location = nemo_file_get_location (file);
+	location = nemo_file_get_activation_location (file);
 	if (g_file_is_native (location)) {
 		res = TRUE;
 	} else {
@@ -328,7 +339,9 @@ nemo_mime_get_default_application_for_file (NemoFile *file)
 	char *uri_scheme;
 
 	if (!nemo_mime_actions_check_if_required_attributes_ready (file)) {
-		return NULL;
+        if (file_has_local_path (file)) {
+            return NULL;
+        }
 	}
 
 	mime_type = nemo_file_get_mime_type (file);
@@ -385,8 +398,8 @@ static int
 application_compare_by_name (const GAppInfo *app_a,
 			     const GAppInfo *app_b)
 {
-	return g_utf8_collate (g_app_info_get_display_name ((GAppInfo *)app_a),
-			       g_app_info_get_display_name ((GAppInfo *)app_b));
+	return g_utf8_collate (g_app_info_get_name ((GAppInfo *)app_a),
+			       g_app_info_get_name ((GAppInfo *)app_b));
 }
 
 static int
@@ -442,7 +455,7 @@ nemo_mime_get_applications_for_file (NemoFile *file)
 		}
 		g_free (uri_scheme);
 	}
-	
+
 	if (!file_has_local_path (file)) {
 		/* Filter out non-uri supporting apps */
 		result = filter_non_uri_apps (result);
@@ -763,7 +776,7 @@ nemo_mime_file_opens_in_view (NemoFile *file)
 }
 
 static ActivationAction
-get_activation_action (NemoFile *file)
+get_activation_action (NemoFile *file, gboolean is_desktop)
 {
 	ActivationAction action;
 	char *activation_uri;
@@ -793,7 +806,7 @@ get_activation_action (NemoFile *file)
 	} 
 
 	if (action == ACTIVATION_ACTION_DO_NOTHING) {
-		if (nemo_mime_file_opens_in_view (file)) {
+		if (nemo_mime_file_opens_in_view (file) && !is_desktop) {
 			action = ACTIVATION_ACTION_OPEN_IN_VIEW;
 		} else {
 			action = ACTIVATION_ACTION_OPEN_IN_APPLICATION;
@@ -809,7 +822,7 @@ nemo_mime_file_opens_in_external_app (NemoFile *file)
 {
   ActivationAction activation_action;
   
-  activation_action = get_activation_action (file);
+  activation_action = get_activation_action (file, FALSE);
   
   return (activation_action == ACTIVATION_ACTION_OPEN_IN_APPLICATION);
 }
@@ -1058,432 +1071,207 @@ confirm_multiple_windows (GtkWindow *parent_window,
 }
 
 typedef struct {
-	NemoWindowSlot *slot;
-	GtkWindow *parent_window;
-	NemoFile *file;
-	GList *files;
-	NemoWindowOpenFlags flags;
-	char *activation_directory;
-	gboolean user_confirmation;
-	char *uri;
-	GDBusProxy *proxy;
-	GtkWidget *dialog;
-} ActivateParametersInstall;
+    GtkWindow *parent_window;
+    NemoFile *file;
+} ActivateParametersSpecial;
 
 static void
-activate_parameters_install_free (ActivateParametersInstall *parameters_install)
+activate_parameters_special_free (ActivateParametersSpecial *parameters_special)
 {
-	if (parameters_install->slot) {
-		g_object_remove_weak_pointer (G_OBJECT (parameters_install->slot), (gpointer *)&parameters_install->slot);
-	}
-	if (parameters_install->parent_window) {
-		g_object_remove_weak_pointer (G_OBJECT (parameters_install->parent_window), (gpointer *)&parameters_install->parent_window);
-	}
-
-	if (parameters_install->proxy != NULL) {
-	        g_object_unref (parameters_install->proxy);
-	}
-
-	nemo_file_unref (parameters_install->file);
-	nemo_file_list_free (parameters_install->files);
-	g_free (parameters_install->activation_directory);
-	g_free (parameters_install->uri);
-	g_free (parameters_install);
-}
-
-static char *
-get_application_no_mime_type_handler_message (NemoFile *file, char *uri)
-{
-	char *uri_for_display;
-	char *nice_uri;
-	char *error_message;
-	GFile *location;
-
-	/* For local files, we want to use filename if possible */
-	if (nemo_file_is_local (file)) {
-		location = nemo_file_get_location (file);
-		nice_uri = g_file_get_parse_name (location);
-		g_object_unref (location);
-	} else {
-		nice_uri = g_strdup (uri);
-	}
-
-	/* Truncate the URI so it doesn't get insanely wide. Note that even
-	 * though the dialog uses wrapped text, if the URI doesn't contain
-	 * white space then the text-wrapping code is too stupid to wrap it.
-	 */
-	uri_for_display = eel_str_middle_truncate (nice_uri, MAX_URI_IN_DIALOG_LENGTH);
-	error_message = g_strdup_printf (_("Could not display \"%s\"."), uri_for_display);
-	g_free (nice_uri);
-	g_free (uri_for_display);
-	return error_message;
+    if (parameters_special->parent_window) {
+        g_object_remove_weak_pointer (G_OBJECT (parameters_special->parent_window), (gpointer *)&parameters_special->parent_window);
+    }
+    nemo_file_unref (parameters_special->file);
+    g_free (parameters_special);
 }
 
 static void
-open_with_response_cb (GtkDialog *dialog,
-		       gint response_id,
-		       gpointer user_data)
+make_exec_callback (NemoFile *file,
+                       GFile *res_loc,
+                      GError *error,
+                     gpointer callback_data)
 {
-	GtkWindow *parent_window;
-	NemoFile *file;
-	GList files;
-	GAppInfo *info;
-	ActivateParametersInstall *parameters = user_data;
-	
-	if (response_id != GTK_RESPONSE_OK) {
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-		return;
-	}
+    ActivateParametersSpecial *params = (ActivateParametersSpecial *) callback_data;
 
-	parent_window = parameters->parent_window;
-	file = g_object_get_data (G_OBJECT (dialog), "mime-action:file");
-	info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (dialog));
+    if (error == NULL) {
+        gchar *path = nemo_file_get_path (params->file);
+        nemo_launch_application_from_command (gtk_widget_get_screen (GTK_WIDGET (params->parent_window)),
+                                              path, FALSE, NULL);
+        g_free (path);
+    }
+    nemo_report_error_setting_permissions (file, error, NULL);
 
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-
-	g_signal_emit_by_name (nemo_signaller_get_current (), "mime_data_changed");
-
-	files.next = NULL;
-	files.prev = NULL;
-	files.data = file;
-	nemo_launch_application (info, &files, parent_window);
-
-	g_object_unref (info);
-
-	activate_parameters_install_free (parameters);
+    activate_parameters_special_free (params);
 }
 
 static void
-choose_program (GtkDialog *message_dialog, int response, gpointer callback_data)
+open_with_dialog_response_cb (GtkDialog *dialog,
+                              gint response_id,
+                              gpointer user_data)
 {
-	GtkWidget *dialog;
-	NemoFile *file;
-	GFile *location;
-	ActivateParametersInstall *parameters = callback_data;
+    GtkWindow *parent_window;
+    NemoFile *file;
+    GAppInfo *info;
+    GList files;
 
-	if (response != GTK_RESPONSE_ACCEPT){
-		gtk_widget_destroy (GTK_WIDGET (message_dialog));
-		activate_parameters_install_free (parameters);
-		return;
-	}
+    parent_window = user_data;
 
-	file = g_object_get_data (G_OBJECT (message_dialog), "mime-action:file");
+    if (response_id != GTK_RESPONSE_OK) {
+        gtk_widget_destroy (GTK_WIDGET (dialog));
+        return;
+    }
 
-	g_assert (NEMO_IS_FILE (file));
+    GtkWidget *content = gtk_dialog_get_content_area (dialog);
+    GList *children = gtk_container_get_children (GTK_CONTAINER (content));
 
-	location = nemo_file_get_location (file);
-	nemo_file_ref (file);
+    NemoMimeApplicationChooser *chooser = children->data;
 
-	/* Destroy the message dialog after ref:ing the file */
-	gtk_widget_destroy (GTK_WIDGET (message_dialog));
+    g_list_free (children);
 
-	dialog = gtk_app_chooser_dialog_new (parameters->parent_window,
-					     0, location);
-	g_object_set_data_full (G_OBJECT (dialog), 
-				"mime-action:file",
-				nemo_file_ref (file),
-				(GDestroyNotify)nemo_file_unref);
+    info = nemo_mime_application_chooser_get_info (chooser);
+    file = nemo_file_get_by_uri (nemo_mime_application_chooser_get_uri (chooser));
 
-	gtk_widget_show (dialog);
+    g_signal_emit_by_name (nemo_signaller_get_current (), "mime_data_changed");
 
-	g_signal_connect (dialog, 
-			  "response", 
-			  G_CALLBACK (open_with_response_cb),
-			  parameters);
+    files.next = NULL;
+    files.prev = NULL;
+    files.data = file;
+    nemo_launch_application (info, &files, parent_window);
 
-	g_object_unref (location);
-	nemo_file_unref (file);	
+    gtk_widget_destroy (GTK_WIDGET (dialog));
+    g_object_unref (info);
 }
 
 static void
-show_unhandled_type_error (ActivateParametersInstall *parameters)
+run_open_with_dialog (ActivateParametersSpecial *params)
 {
-	GtkWidget *dialog;
+    GtkWidget *dialog;
+    GtkWidget *ok_button;
 
-	char *mime_type = nemo_file_get_mime_type (parameters->file);
-	char *error_message = get_application_no_mime_type_handler_message (parameters->file, parameters->uri);
-	if (g_content_type_is_unknown (mime_type)) {
-		dialog = gtk_message_dialog_new (parameters->parent_window,
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 0,
-						 NULL);
-		g_object_set (dialog,
-			      "text", error_message,
-			      "secondary-text", _("The file is of an unknown type"),
-			      NULL);
-	} else {
-		char *text;
-		text = g_strdup_printf (_("There is no application installed for %s files"), g_content_type_get_description (mime_type));
+    char *mime_type;
+    char *uri = NULL;
+    GList *uris = NULL;
 
-		dialog = gtk_message_dialog_new (parameters->parent_window,
-						 GTK_DIALOG_DESTROY_WITH_PARENT,
-						 GTK_MESSAGE_ERROR,
-						 0,
-						 NULL);
-		g_object_set (dialog,
-			      "text", error_message,
-			      "secondary-text", text,
-			      NULL);
+    mime_type = nemo_file_get_mime_type (params->file);
+    uri = nemo_file_get_uri (params->file);
 
-		g_free (text);
-	}
+    dialog = gtk_dialog_new_with_buttons (_("Open with"),
+                                          params->parent_window,
+                                          GTK_DIALOG_DESTROY_WITH_PARENT,
+                                          GTK_STOCK_CANCEL,
+                                          GTK_RESPONSE_CANCEL,
+                                          NULL);
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Select Application"), GTK_RESPONSE_ACCEPT);
+    ok_button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+                                       GTK_STOCK_OK,
+                                       GTK_RESPONSE_OK);
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+    GtkWidget *chooser = nemo_mime_application_chooser_new (uri, uris, mime_type, ok_button);
 
-	g_object_set_data_full (G_OBJECT (dialog), 
-				"mime-action:file",
-				nemo_file_ref (parameters->file),
-				(GDestroyNotify)nemo_file_unref);
+    GtkWidget *content = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
 
-	gtk_widget_show (GTK_WIDGET (dialog));
-	
-	g_signal_connect (dialog, "response",
-			  G_CALLBACK (choose_program), parameters);
+    gtk_box_pack_start (GTK_BOX (content), chooser, TRUE, TRUE, 0);
 
-	g_free (error_message);
-	g_free (mime_type);
+    gtk_widget_show_all (dialog);
+
+    g_signal_connect_object (dialog, "response", 
+                             G_CALLBACK (open_with_dialog_response_cb),
+                             params->parent_window, 0);
+    activate_parameters_special_free (params);
 }
 
 static void
-search_for_application_dbus_call_notify_cb (GDBusProxy   *proxy,
-					    GAsyncResult *result,
-					    gpointer      user_data)
+unhandled_uri_response_callback (GtkDialog *dialog,
+                                 int response_id,
+                                 ActivateParametersSpecial *parameters)
 {
-	ActivateParametersInstall *parameters_install = user_data;
-	GVariant *variant;
-	GError *error = NULL;
-
-	variant = g_dbus_proxy_call_finish (proxy, result, &error);
-	if (variant == NULL) {
-		if (!g_dbus_error_is_remote_error (error) ||
-		    g_strcmp0 (g_dbus_error_get_remote_error (error), "org.freedesktop.PackageKit.Modify.Failed") == 0) {
-			    char *message;
-
-			    message = g_strdup_printf ("%s\n%s",
-						       _("There was an internal error trying to search for applications:"),
-						       error->message);
-			    eel_show_error_dialog (_("Unable to search for application"), message,
-			                           parameters_install->parent_window);
-			    g_free (message);
-		    }
-
-		g_error_free (error);
-		activate_parameters_install_free (parameters_install);
-		return;
-	}
-
-	g_variant_unref (variant);
-
-	/* activate the file again */
-	nemo_mime_activate_files (parameters_install->parent_window,
-	                              parameters_install->slot,
-	                              parameters_install->files,
-	                              parameters_install->activation_directory,
-	                              parameters_install->flags,
-	                              parameters_install->user_confirmation);
-
-	activate_parameters_install_free (parameters_install);
-}
-
-static void
-search_for_application_mime_type (ActivateParametersInstall *parameters_install, const gchar *mime_type)
-{
-	GdkWindow *window;
-	guint xid = 0;
-	const char *mime_types[2];
-
-	g_assert (parameters_install->proxy != NULL);	
-
-	/* get XID from parent window */
-	window = gtk_widget_get_window (GTK_WIDGET (parameters_install->parent_window));
-	if (window != NULL) {
-		xid = GDK_WINDOW_XID (window);
-	}
-
-	mime_types[0] = mime_type;
-	mime_types[1] = NULL;
-
-	g_dbus_proxy_call (parameters_install->proxy,
-			   "InstallMimeTypes",
-			   g_variant_new ("(u^ass)",
-					  xid,
-					  mime_types,
-					  "hide-confirm-search"),
-			   G_DBUS_CALL_FLAGS_NONE,
-			   G_MAXINT /* no timeout */,
-			   NULL /* cancellable */,
-			   (GAsyncReadyCallback) search_for_application_dbus_call_notify_cb,
-			   parameters_install);
-
-	DEBUG ("InstallMimeType method invoked for %s", mime_type);
-}
-
-static void
-application_unhandled_file_install (GtkDialog *dialog,
-                                    gint response_id,
-                                    ActivateParametersInstall *parameters_install)
-{
-	char *mime_type;
-
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	parameters_install->dialog = NULL;
-
-	if (response_id == GTK_RESPONSE_YES) {
-		mime_type = nemo_file_get_mime_type (parameters_install->file);
-		search_for_application_mime_type (parameters_install, mime_type);
-		g_free (mime_type);
-	} else {
-		/* free as we're not going to get the async dbus callback */
-		activate_parameters_install_free (parameters_install);
-	}
-}
-
-static gboolean
-delete_cb (GtkDialog *dialog)
-{
-	gtk_dialog_response (dialog, GTK_RESPONSE_DELETE_EVENT);
-	return TRUE;
-}
-
-static void
-pk_proxy_appeared_cb (GObject *source,
-		      GAsyncResult *res,
-		      gpointer user_data)
-{
-        ActivateParametersInstall *parameters_install = user_data;
-	char *mime_type;
-	char *error_message;
-	GtkWidget *dialog;
-        GDBusProxy *proxy;
-	GError *error = NULL;
-
-	proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
-
-	if (error != NULL) {
-		g_warning ("Couldn't call Modify on the PackageKit interface: %s",
-			   error->message);
-		g_error_free (error);
-
-		/* show an unhelpful dialog */
-		show_unhandled_type_error (parameters_install);
-		/* The callback wasn't started, so we have to free the parameters */
-		activate_parameters_install_free (parameters_install);
-
-		return;
-	}
-
-	mime_type = nemo_file_get_mime_type (parameters_install->file);
-	error_message = get_application_no_mime_type_handler_message (parameters_install->file,
-	                                                              parameters_install->uri);
-	/* use a custom dialog to prompt the user to install new software */
-	dialog = gtk_message_dialog_new (parameters_install->parent_window, 0,
-	                                 GTK_MESSAGE_ERROR,
-	                                 GTK_BUTTONS_YES_NO,
-	                                 "%s", error_message);
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-	                                          _("There is no application installed for %s files.\n"
-	                                            "Do you want to search for an application to open this file?"),
-	                                          g_content_type_get_description (mime_type));
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-
-	parameters_install->dialog = dialog;
-	parameters_install->proxy = proxy;
-
-	g_signal_connect (dialog, "response",
-	                  G_CALLBACK (application_unhandled_file_install),
-	                  parameters_install);
-	g_signal_connect (dialog, "delete-event",
-	                  G_CALLBACK (delete_cb), NULL);
-	gtk_widget_show_all (dialog);
-	g_free (mime_type);
+    switch (response_id) {
+    case RESPONSE_RUN:
+        ;
+        guint32 existing = nemo_file_get_permissions (parameters->file);
+        nemo_file_set_permissions (parameters->file,
+                                   existing | (UNIX_PERM_USER_EXEC|UNIX_PERM_GROUP_EXEC|UNIX_PERM_OTHER_EXEC),
+                                   (NemoFileOperationCallback) make_exec_callback,
+                                   parameters);
+        break;
+    case RESPONSE_OPEN_WITH:
+        run_open_with_dialog (parameters);
+        break;
+    default:
+        /* Just destroy dialog */
+        break;
+    }
+    gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static void
 application_unhandled_uri (ActivateParameters *parameters, char *uri)
 {
-	gboolean show_install_mime;
-	char *mime_type;
-	NemoFile *file;
-	ActivateParametersInstall *parameters_install;
 
-	file = nemo_file_get_by_uri (uri);
+    NemoFile *file = nemo_file_get_existing_by_uri (uri);
 
-	mime_type = nemo_file_get_mime_type (file);
+    char *primary, *secondary, *display_name;
+    GtkWidget *dialog;
 
-	/* copy the parts of parameters we are interested in as the orignal will be unref'd */
-	parameters_install = g_new0 (ActivateParametersInstall, 1);
-	parameters_install->slot = parameters->slot;
-	g_object_add_weak_pointer (G_OBJECT (parameters_install->slot), (gpointer *)&parameters_install->slot);
-	if (parameters->parent_window) {
-		parameters_install->parent_window = parameters->parent_window;
-		g_object_add_weak_pointer (G_OBJECT (parameters_install->parent_window), (gpointer *)&parameters_install->parent_window);
-	}
-	parameters_install->activation_directory = g_strdup (parameters->activation_directory);
-	parameters_install->file = file;
-	parameters_install->files = get_file_list_for_launch_locations (parameters->locations);
-	parameters_install->flags = parameters->flags;
-	parameters_install->user_confirmation = parameters->user_confirmation;
-	parameters_install->uri = g_strdup(uri);
+    ActivateParametersSpecial *parameters_special;
 
-#ifdef ENABLE_PACKAGEKIT
-	/* allow an admin to disable the PackageKit search functionality */
-	show_install_mime = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_INSTALL_MIME_ACTIVATION);
-#else
-	/* we have no install functionality */
-	show_install_mime = FALSE;
-#endif
-	/* There is no use trying to look for handlers of application/octet-stream */
-	if (g_content_type_is_unknown (mime_type)) {
-		show_install_mime = FALSE;
-	}
+    parameters_special = g_new0 (ActivateParametersSpecial, 1);
+    if (parameters->parent_window) {
+        parameters_special->parent_window = parameters->parent_window;
+        g_object_add_weak_pointer (G_OBJECT (parameters_special->parent_window), (gpointer *)&parameters_special->parent_window);
+    }
+    parameters_special->file = nemo_file_ref (file);
 
-	g_free (mime_type);
+    primary = _("Unknown file type");
+    display_name = nemo_file_get_display_name (file);
+    secondary =
+        g_strdup_printf (_("The file \"%s\" has no known programs associated with it.  "
+                   "If you trust the source of this file, and have sufficient permissions, you can mark it executable and launch it.  "
+                   "Or, you can use the Open With dialog to pick a program to associate it with."
+                   ),
+                 display_name);
+    
+    dialog = gtk_message_dialog_new (parameters->parent_window,
+                     0,
+                     GTK_MESSAGE_WARNING,
+                     GTK_BUTTONS_NONE,
+                     NULL);
+    g_object_set (dialog,
+              "text", primary,
+              "secondary-text", secondary,
+              NULL);
 
-	if (!show_install_mime) {
-		goto out;
-	}
+    gtk_dialog_add_button (GTK_DIALOG (dialog),
+                   _("Make executable and run"), RESPONSE_RUN);
+    gtk_dialog_add_button (GTK_DIALOG (dialog),
+                   _("Choose a program"), RESPONSE_OPEN_WITH);
 
-	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
-				  G_DBUS_PROXY_FLAGS_NONE,
-				  NULL,
-				  "org.freedesktop.PackageKit",
-				  "/org/freedesktop/PackageKit",
-				  "org.freedesktop.PackageKit.Modify",
-				  NULL,
-				  pk_proxy_appeared_cb,
-				  parameters_install);
+    if (!nemo_file_can_set_permissions (file)) {
+        GtkWidget *w = gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), RESPONSE_RUN);
+        gtk_widget_set_sensitive (w, FALSE);
+    }
 
-	return;
+    gtk_dialog_add_button (GTK_DIALOG (dialog),
+                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
-out:
-        /* show an unhelpful dialog */
-        show_unhandled_type_error (parameters_install);
+    g_signal_connect (dialog, "response",
+              G_CALLBACK (unhandled_uri_response_callback),
+                          parameters_special);
+    gtk_widget_show (dialog);
+    
+    g_free (display_name);
+    g_free (secondary);
+    return;
 }
 
-typedef struct {
-	GtkWindow *parent_window;
-	NemoFile *file;
-} ActivateParametersDesktop;
-
-static void
-activate_parameters_desktop_free (ActivateParametersDesktop *parameters_desktop)
-{
-	if (parameters_desktop->parent_window) {
-		g_object_remove_weak_pointer (G_OBJECT (parameters_desktop->parent_window), (gpointer *)&parameters_desktop->parent_window);
-	}
-	nemo_file_unref (parameters_desktop->file);
-	g_free (parameters_desktop);
-}
 
 static void
 untrusted_launcher_response_callback (GtkDialog *dialog,
 				      int response_id,
-				      ActivateParametersDesktop *parameters)
+				      ActivateParametersSpecial *parameters)
 {
 	GdkScreen *screen;
 	char *uri;
@@ -1512,14 +1300,14 @@ untrusted_launcher_response_callback (GtkDialog *dialog,
 	}
 	
 	gtk_widget_destroy (GTK_WIDGET (dialog));
-	activate_parameters_desktop_free (parameters);
+	activate_parameters_special_free (parameters);
 }
 
 static void
 activate_desktop_file (ActivateParameters *parameters,
 		       NemoFile *file)
 {
-	ActivateParametersDesktop *parameters_desktop;
+	ActivateParametersSpecial *parameters_special;
 	char *primary, *secondary, *display_name;
 	GtkWidget *dialog;
 	GdkScreen *screen;
@@ -1529,17 +1317,17 @@ activate_desktop_file (ActivateParameters *parameters,
 
 	if (!nemo_file_is_trusted_link (file)) {
 		/* copy the parts of parameters we are interested in as the orignal will be freed */
-		parameters_desktop = g_new0 (ActivateParametersDesktop, 1);
+		parameters_special = g_new0 (ActivateParametersSpecial, 1);
 		if (parameters->parent_window) {
-			parameters_desktop->parent_window = parameters->parent_window;
-			g_object_add_weak_pointer (G_OBJECT (parameters_desktop->parent_window), (gpointer *)&parameters_desktop->parent_window);
+			parameters_special->parent_window = parameters->parent_window;
+			g_object_add_weak_pointer (G_OBJECT (parameters_special->parent_window), (gpointer *)&parameters_special->parent_window);
 		}
-		parameters_desktop->file = nemo_file_ref (file);
+		parameters_special->file = nemo_file_ref (file);
 
 		primary = _("Untrusted application launcher");
 		display_name = nemo_file_get_display_name (file);
 		secondary =
-			g_strdup_printf (_("The application launcher \"%s\" has not been marked as trusted. "
+			g_strdup_printf (_("The application launcher \"%s\" has not been marked as trusted (executable). "
 					   "If you do not know the source of this file, launching it may be unsafe."
 					   ),
 					 display_name);
@@ -1553,21 +1341,21 @@ activate_desktop_file (ActivateParameters *parameters,
 			      "text", primary,
 			      "secondary-text", secondary,
 			      NULL);
-		if (nemo_file_get_ctime(file) < 1256817600L ) {
+
 		gtk_dialog_add_button (GTK_DIALOG (dialog),
 				       _("_Launch Anyway"), RESPONSE_RUN);
 		if (nemo_file_can_set_permissions (file)) {
 			gtk_dialog_add_button (GTK_DIALOG (dialog),
 					       _("Mark as _Trusted"), RESPONSE_MARK_TRUSTED);
 		}
-		}
+
 		gtk_dialog_add_button (GTK_DIALOG (dialog),
 				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
 		g_signal_connect (dialog, "response",
 				  G_CALLBACK (untrusted_launcher_response_callback),
-				  parameters_desktop);
+				  parameters_special);
 		gtk_widget_show (dialog);
 		
 		g_free (display_name);
@@ -1608,15 +1396,25 @@ activate_files (ActivateParameters *parameters)
 	gint num_unhandled;
 	gint num_files;
 	gboolean open_files;
-	
+    gboolean launch_location_is_desktop;
+
 	screen = gtk_widget_get_screen (GTK_WIDGET (parameters->parent_window));
 
 	launch_desktop_files = NULL;
 	launch_files = NULL;
 	launch_in_terminal_files = NULL;
+    launch_location_is_desktop = FALSE;
 	open_in_app_uris = NULL;
 	open_in_view_files = NULL;
 
+    window = NULL;
+
+    if (parameters->slot != NULL) {
+        window = nemo_window_slot_get_window (parameters->slot);
+
+        launch_location_is_desktop = NEMO_IS_DESKTOP_WINDOW (window);
+    }
+
 	for (l = parameters->locations; l != NULL; l = l->next) {
 		location = l->data;
 		file = location->file;
@@ -1625,7 +1423,7 @@ activate_files (ActivateParameters *parameters)
 			continue;
 		}
 
-		action = get_activation_action (file);
+		action = get_activation_action (file, launch_location_is_desktop);
 		if (action == ACTIVATION_ACTION_ASK) {
 			/* Special case for executable text files, since it might be
 			 * dangerous & unexpected to launch these.
@@ -1718,7 +1516,8 @@ activate_files (ActivateParameters *parameters)
 
 	flags = parameters->flags;
 	if (count > 1) {
-		if ((parameters->flags & NEMO_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0) {
+		if ((parameters->flags & NEMO_WINDOW_OPEN_FLAG_NEW_WINDOW) == 0 &&
+            g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_ALWAYS_USE_BROWSER)) {
 			flags |= NEMO_WINDOW_OPEN_FLAG_NEW_TAB;
 		} else {
 			flags |= NEMO_WINDOW_OPEN_FLAG_NEW_WINDOW;
@@ -1750,8 +1549,7 @@ activate_files (ActivateParameters *parameters)
 
 			uri = nemo_file_get_activation_uri (file);
 			f = g_file_new_for_uri (uri);
-			nemo_window_slot_open_location (parameters->slot,
-							    f, flags, NULL);
+            nemo_window_slot_open_location (parameters->slot, f, flags);
 			g_object_unref (f);
 			g_free (uri);
 		}
@@ -1819,11 +1617,6 @@ activate_files (ActivateParameters *parameters)
 		}
 	}
 
-	window = NULL;
-	if (parameters->slot != NULL) {
-		window = nemo_window_slot_get_window (parameters->slot);
-	}
-
 	if (open_in_app_parameters != NULL ||
 	    unhandled_open_in_app_uris != NULL) {
 		if ((parameters->flags & NEMO_WINDOW_OPEN_FLAG_CLOSE_BEHIND) != 0 &&
@@ -2359,3 +2152,18 @@ nemo_mime_activate_file (GtkWindow *parent_window,
 	nemo_mime_activate_files (parent_window, slot, files, launch_directory, flags, FALSE);
 	g_list_free (files);
 }
+
+void
+nemo_mime_launch_fm_and_select_file (GFile *file)
+{
+    GAppInfo *info;
+    GList *list;
+
+    info = g_app_info_get_default_for_type ("inode/directory", !g_file_is_native (file));
+
+    list = g_list_prepend (NULL, file);
+
+    g_app_info_launch (info, list, NULL, NULL);
+
+    g_list_free (list);
+}
